@@ -303,8 +303,8 @@ heif_error loadJPEG(const char *filename, InputImage *input_image)
     err = heif_image_add_plane(image, heif_channel_Y, cinfo.output_width, cinfo.output_height, 8);
     if (err.code) { goto cleanup; }
 
-    int y_stride;
-    uint8_t* py = heif_image_get_plane(image, heif_channel_Y, &y_stride);
+    size_t y_stride;
+    uint8_t* py = heif_image_get_plane2(image, heif_channel_Y, &y_stride);
 
 
     // read the image
@@ -385,11 +385,11 @@ heif_error loadJPEG(const char *filename, InputImage *input_image)
     err = heif_image_add_plane(image, heif_channel_Cr, cw, ch, 8);
     if (err.code) { goto cleanup; }
 
-    int stride[3];
+    size_t stride[3];
     uint8_t* p[3];
-    p[0] = heif_image_get_plane(image, heif_channel_Y, &stride[0]);
-    p[1] = heif_image_get_plane(image, heif_channel_Cb, &stride[1]);
-    p[2] = heif_image_get_plane(image, heif_channel_Cr, &stride[2]);
+    p[0] = heif_image_get_plane2(image, heif_channel_Y, &stride[0]);
+    p[1] = heif_image_get_plane2(image, heif_channel_Cb, &stride[1]);
+    p[2] = heif_image_get_plane2(image, heif_channel_Cr, &stride[2]);
 
     // read the image
 
