@@ -23,7 +23,7 @@
 #include "avif_dec.h"
 #include "bitstream.h"
 #include "common_utils.h"
-#include "libheif/api_structs.h"
+#include "api_structs.h"
 #include "file.h"
 #include <iomanip>
 #include <limits>
