@@ -32,13 +32,35 @@ public:
   std::vector<ColorStateWithCost>
   state_after_conversion(const ColorState& input_state,
                          const ColorState& target_state,
-                         const heif_color_conversion_options& options) const override;
+                         const heif_color_conversion_options& options,
+                         const heif_color_conversion_options_ext& options_ext) const override;
 
   Result<std::shared_ptr<HeifPixelImage>>
   convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                      const ColorState& input_state,
                      const ColorState& target_state,
                      const heif_color_conversion_options& options,
+                     const heif_color_conversion_options_ext& options_ext,
+                     const heif_security_limits* limits) const override;
+};
+
+
+template<class Pixel>
+class Op_flatten_alpha_plane : public ColorConversionOperation
+{
+public:
+  std::vector<ColorStateWithCost>
+  state_after_conversion(const ColorState& input_state,
+                         const ColorState& target_state,
+                         const heif_color_conversion_options& options,
+                         const heif_color_conversion_options_ext& options_ext) const override;
+
+  Result<std::shared_ptr<HeifPixelImage>>
+  convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
+                     const ColorState& input_state,
+                     const ColorState& target_state,
+                     const heif_color_conversion_options& options,
+                     const heif_color_conversion_options_ext& options_ext,
                      const heif_security_limits* limits) const override;
 };
 
