@@ -28,7 +28,8 @@
 std::vector<ColorStateWithCost>
 Op_RGB_to_RGB24_32::state_after_conversion(const ColorState& input_state,
                                            const ColorState& target_state,
-                                           const heif_color_conversion_options& options) const
+                                           const heif_color_conversion_options& options,
+                                           const heif_color_conversion_options_ext& options_ext) const
 {
   if (input_state.colorspace != heif_colorspace_RGB ||
       input_state.chroma != heif_chroma_444 ||
@@ -48,7 +49,7 @@ Op_RGB_to_RGB24_32::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = true;
   output_state.bits_per_pixel = 8;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
   // --- convert to RGB (without alpha)
 
@@ -57,17 +58,19 @@ Op_RGB_to_RGB24_32::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = false;
   output_state.bits_per_pixel = 8;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RGB_to_RGB24_32::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                        const ColorState& input_state,
                                        const ColorState& target_state,
-                                       const heif_color_conversion_options& options) const
+                                       const heif_color_conversion_options& options,
+                                       const heif_color_conversion_options_ext& options_ext,
+                                       const heif_security_limits* limits) const
 {
   bool has_alpha = input->has_channel(heif_channel_Alpha);
   bool want_alpha = target_state.has_alpha;
@@ -75,30 +78,30 @@ Op_RGB_to_RGB24_32::convert_colorspace(const std::shared_ptr<const HeifPixelImag
   if (input->get_bits_per_pixel(heif_channel_R) != 8 ||
       input->get_bits_per_pixel(heif_channel_G) != 8 ||
       input->get_bits_per_pixel(heif_channel_B) != 8) {
-    return nullptr;
+    return Error::InternalError;
   }
 
   if (has_alpha && input->get_bits_per_pixel(heif_channel_Alpha) != 8) {
-    return nullptr;
+    return Error::InternalError;
   }
 
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
 
   outimg->create(width, height, heif_colorspace_RGB,
                  want_alpha ? heif_chroma_interleaved_32bit : heif_chroma_interleaved_24bit);
 
-  if (!outimg->add_plane(heif_channel_interleaved, width, height, 8)) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_interleaved, width, height, 8, limits)) {
+    return err;
   }
 
   const uint8_t* in_r, * in_g, * in_b, * in_a = nullptr;
-  int in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
+  size_t in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
 
   uint8_t* out_p;
-  int out_p_stride = 0;
+  size_t out_p_stride = 0;
 
   in_r = input->get_plane(heif_channel_R, &in_r_stride);
   in_g = input->get_plane(heif_channel_G, &in_g_stride);
@@ -109,7 +112,7 @@ Op_RGB_to_RGB24_32::convert_colorspace(const std::shared_ptr<const HeifPixelImag
     in_a = input->get_plane(heif_channel_Alpha, &in_a_stride);
   }
 
-  int x, y;
+  uint32_t x, y;
   for (y = 0; y < height; y++) {
 
     if (has_alpha && want_alpha) {
@@ -146,13 +149,14 @@ Op_RGB_to_RGB24_32::convert_colorspace(const std::shared_ptr<const HeifPixelImag
 std::vector<ColorStateWithCost>
 Op_RGB_HDR_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
                                                   const ColorState& target_state,
-                                                  const heif_color_conversion_options& options) const
+                                                  const heif_color_conversion_options& options,
+                                                  const heif_color_conversion_options_ext& options_ext) const
 {
   // Note: no input alpha channel required. It will be filled up with 0xFF.
 
   if (input_state.colorspace != heif_colorspace_RGB ||
       input_state.chroma != heif_chroma_444 ||
-      input_state.bits_per_pixel == 8) {
+      input_state.bits_per_pixel <= 8) {
     return {};
   }
 
@@ -168,7 +172,7 @@ Op_RGB_HDR_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
     output_state.has_alpha = false;
     output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-    states.push_back({output_state, SpeedCosts_Unoptimized});
+    states.emplace_back(output_state, SpeedCosts_Unoptimized);
   }
 
 
@@ -179,57 +183,59 @@ Op_RGB_HDR_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = true;
   output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
 
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RGB_HDR_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                               const ColorState& input_state,
                                               const ColorState& target_state,
-                                              const heif_color_conversion_options& options) const
+                                              const heif_color_conversion_options& options,
+                                              const heif_color_conversion_options_ext& options_ext,
+                                              const heif_security_limits* limits) const
 {
-  if (input->get_bits_per_pixel(heif_channel_R) == 8 ||
-      input->get_bits_per_pixel(heif_channel_G) == 8 ||
-      input->get_bits_per_pixel(heif_channel_B) == 8) {
-    return nullptr;
+  if (input->get_bits_per_pixel(heif_channel_R) <= 8 ||
+      input->get_bits_per_pixel(heif_channel_G) <= 8 ||
+      input->get_bits_per_pixel(heif_channel_B) <= 8) {
+    return Error::InternalError;
   }
 
   bool input_has_alpha = input->has_channel(heif_channel_Alpha);
   bool output_has_alpha = input_has_alpha || target_state.has_alpha;
 
   if (input_has_alpha) {
-    if (input->get_bits_per_pixel(heif_channel_Alpha) == 8) {
-      return nullptr;
+    if (input->get_bits_per_pixel(heif_channel_Alpha) <= 8) {
+      return Error::InternalError;
     }
 
     if (input->get_width(heif_channel_Alpha) != input->get_width(heif_channel_G) ||
         input->get_height(heif_channel_Alpha) != input->get_height(heif_channel_G)) {
-      return nullptr;
+      return Error::InternalError;
     }
   }
   int bpp = input->get_bits_per_pixel(heif_channel_R);
-  if (bpp <= 0) return nullptr;
+  if (bpp <= 0) return Error::InternalError;
 
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
 
   outimg->create(width, height, heif_colorspace_RGB,
                  output_has_alpha ? heif_chroma_interleaved_RRGGBBAA_BE : heif_chroma_interleaved_RRGGBB_BE);
-  if (!outimg->add_plane(heif_channel_interleaved, width, height, bpp)) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_interleaved, width, height, bpp, limits)) {
+    return err;
   }
 
   const uint16_t* in_r, * in_g, * in_b, * in_a = nullptr;
-  int in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
+  size_t in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
 
   uint8_t* out_p;
-  int out_p_stride = 0;
+  size_t out_p_stride = 0;
 
   in_r = (uint16_t*) input->get_plane(heif_channel_R, &in_r_stride);
   in_g = (uint16_t*) input->get_plane(heif_channel_G, &in_g_stride);
@@ -238,6 +244,12 @@ Op_RGB_HDR_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPi
 
   if (input_has_alpha) {
     in_a = (uint16_t*) input->get_plane(heif_channel_Alpha, &in_a_stride);
+    assert(in_a != nullptr);
+
+    // should never happen, but makes clang-tidy happy
+    if (in_a == nullptr) {
+      return Error::InternalError;
+    }
   }
 
   in_r_stride /= 2;
@@ -247,8 +259,8 @@ Op_RGB_HDR_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPi
 
   const int pixelsize = (output_has_alpha ? 8 : 6);
 
-  int x, y;
-  uint16_t alpha_max = static_cast<uint16_t>((1 << bpp) - 1);
+  uint32_t x, y;
+  auto alpha_max = static_cast<uint16_t>((1 << bpp) - 1);
   for (y = 0; y < height; y++) {
     for (x = 0; x < width; x++) {
       uint16_t r = in_r[x + y * in_r_stride];
@@ -275,7 +287,8 @@ Op_RGB_HDR_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPi
 std::vector<ColorStateWithCost>
 Op_RGB_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
                                               const ColorState& target_state,
-                                              const heif_color_conversion_options& options) const
+                                              const heif_color_conversion_options& options,
+                                              const heif_color_conversion_options_ext& options_ext) const
 {
   // Note: no input alpha channel required. It will be filled up with 0xFF.
 
@@ -297,7 +310,7 @@ Op_RGB_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
     output_state.has_alpha = false;
     output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-    states.push_back({output_state, SpeedCosts_Unoptimized});
+    states.emplace_back(output_state, SpeedCosts_Unoptimized);
   }
 
 
@@ -308,23 +321,24 @@ Op_RGB_to_RRGGBBaa_BE::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = true;
   output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
-
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RGB_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                           const ColorState& input_state,
                                           const ColorState& target_state,
-                                          const heif_color_conversion_options& options) const
+                                          const heif_color_conversion_options& options,
+                                          const heif_color_conversion_options_ext& options_ext,
+                                          const heif_security_limits* limits) const
 {
   if (input->get_bits_per_pixel(heif_channel_R) != 8 ||
       input->get_bits_per_pixel(heif_channel_G) != 8 ||
       input->get_bits_per_pixel(heif_channel_B) != 8) {
-    return nullptr;
+    return Error::InternalError;
   }
 
   //int bpp = input->get_bits_per_pixel(heif_channel_R);
@@ -333,26 +347,26 @@ Op_RGB_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPixelI
   bool output_has_alpha = input_has_alpha || target_state.has_alpha;
 
   if (input_has_alpha && input->get_bits_per_pixel(heif_channel_Alpha) != 8) {
-    return nullptr;
+    return Error::InternalError;
   }
 
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
 
   outimg->create(width, height, heif_colorspace_RGB,
                  output_has_alpha ? heif_chroma_interleaved_RRGGBBAA_BE : heif_chroma_interleaved_RRGGBB_BE);
 
-  if (!outimg->add_plane(heif_channel_interleaved, width, height, input->get_bits_per_pixel(heif_channel_R))) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_interleaved, width, height, input->get_bits_per_pixel(heif_channel_R), limits)) {
+    return err;
   }
 
   const uint8_t* in_r, * in_g, * in_b, * in_a = nullptr;
-  int in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
+  size_t in_r_stride = 0, in_g_stride = 0, in_b_stride = 0, in_a_stride = 0;
 
   uint8_t* out_p;
-  int out_p_stride = 0;
+  size_t out_p_stride = 0;
 
   in_r = input->get_plane(heif_channel_R, &in_r_stride);
   in_g = input->get_plane(heif_channel_G, &in_g_stride);
@@ -365,7 +379,7 @@ Op_RGB_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPixelI
 
   const int pixelsize = (output_has_alpha ? 8 : 6);
 
-  int x, y;
+  uint32_t x, y;
   for (y = 0; y < height; y++) {
 
     if (input_has_alpha) {
@@ -404,14 +418,15 @@ Op_RGB_to_RRGGBBaa_BE::convert_colorspace(const std::shared_ptr<const HeifPixelI
 std::vector<ColorStateWithCost>
 Op_RRGGBBaa_BE_to_RGB_HDR::state_after_conversion(const ColorState& input_state,
                                                   const ColorState& target_state,
-                                                  const heif_color_conversion_options& options) const
+                                                  const heif_color_conversion_options& options,
+                                                  const heif_color_conversion_options_ext& options_ext) const
 {
   // Note: no input alpha channel required. It will be filled up with 0xFF.
 
   if (input_state.colorspace != heif_colorspace_RGB ||
       (input_state.chroma != heif_chroma_interleaved_RRGGBB_BE &&
        input_state.chroma != heif_chroma_interleaved_RRGGBBAA_BE) ||
-      input_state.bits_per_pixel == 8) {
+      input_state.bits_per_pixel <= 8) {
     return {};
   }
 
@@ -426,18 +441,19 @@ Op_RRGGBBaa_BE_to_RGB_HDR::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = target_state.has_alpha;
   output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
-
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RRGGBBaa_BE_to_RGB_HDR::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                               const ColorState& input_state,
                                               const ColorState& target_state,
-                                              const heif_color_conversion_options& options) const
+                                              const heif_color_conversion_options& options,
+                                              const heif_color_conversion_options_ext& options_ext,
+                                              const heif_security_limits* limits) const
 {
   bool has_alpha = (input->get_chroma_format() == heif_chroma_interleaved_RRGGBBAA_LE ||
                     input->get_chroma_format() == heif_chroma_interleaved_RRGGBBAA_BE);
@@ -445,30 +461,30 @@ Op_RRGGBBaa_BE_to_RGB_HDR::convert_colorspace(const std::shared_ptr<const HeifPi
 
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
   int bpp = input->get_bits_per_pixel(heif_channel_interleaved);
 
   outimg->create(width, height, heif_colorspace_RGB, heif_chroma_444);
 
-  if (!outimg->add_plane(heif_channel_R, width, height, bpp) ||
-      !outimg->add_plane(heif_channel_G, width, height, bpp) ||
-      !outimg->add_plane(heif_channel_B, width, height, bpp)) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_R, width, height, bpp, limits) ||
+                 outimg->add_plane(heif_channel_G, width, height, bpp, limits) ||
+                 outimg->add_plane(heif_channel_B, width, height, bpp, limits)) {
+    return err;
   }
 
   if (want_alpha) {
-    if (!outimg->add_plane(heif_channel_Alpha, width, height, bpp)) {
-      return nullptr;
+    if (auto err = outimg->add_plane(heif_channel_Alpha, width, height, bpp, limits)) {
+      return err;
     }
   }
 
   const uint8_t* in_p;
-  int in_p_stride = 0;
+  size_t in_p_stride = 0;
   int in_pix_size = has_alpha ? 8 : 6;
 
   uint16_t* out_r, * out_g, * out_b, * out_a = nullptr;
-  int out_r_stride = 0, out_g_stride = 0, out_b_stride = 0, out_a_stride = 0;
+  size_t out_r_stride = 0, out_g_stride = 0, out_b_stride = 0, out_a_stride = 0;
 
   in_p = input->get_plane(heif_channel_interleaved, &in_p_stride);
 
@@ -485,8 +501,8 @@ Op_RRGGBBaa_BE_to_RGB_HDR::convert_colorspace(const std::shared_ptr<const HeifPi
   out_b_stride /= 2;
   out_a_stride /= 2;
 
-  uint16_t alpha_max = static_cast<uint16_t>((1 << bpp) - 1);
-  int x, y;
+  auto alpha_max = static_cast<uint16_t>((1 << bpp) - 1);
+  uint32_t x, y;
   for (y = 0; y < height; y++) {
 
     for (x = 0; x < width; x++) {
@@ -517,8 +533,9 @@ Op_RRGGBBaa_BE_to_RGB_HDR::convert_colorspace(const std::shared_ptr<const HeifPi
 
 std::vector<ColorStateWithCost>
 Op_RGB24_32_to_RGB::state_after_conversion(const ColorState& input_state,
-                                                  const ColorState& target_state,
-                                                  const heif_color_conversion_options& options) const
+                                           const ColorState& target_state,
+                                           const heif_color_conversion_options& options,
+                                           const heif_color_conversion_options_ext& options_ext) const
 {
   // Note: no input alpha channel required. It will be filled up with 0xFF.
 
@@ -540,46 +557,48 @@ Op_RGB24_32_to_RGB::state_after_conversion(const ColorState& input_state,
   output_state.has_alpha = target_state.has_alpha;
   output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-  states.push_back({output_state, SpeedCosts_Unoptimized});
+  states.emplace_back(output_state, SpeedCosts_Unoptimized);
 
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RGB24_32_to_RGB::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                        const ColorState& input_state,
-                                              const ColorState& target_state,
-                                              const heif_color_conversion_options& options) const
+                                       const ColorState& target_state,
+                                       const heif_color_conversion_options& options,
+                                       const heif_color_conversion_options_ext& options_ext,
+                                       const heif_security_limits* limits) const
 {
   bool has_alpha = input->get_chroma_format() == heif_chroma_interleaved_RGBA;
   bool want_alpha = target_state.has_alpha;
 
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
 
   outimg->create(width, height, heif_colorspace_RGB, heif_chroma_444);
 
-  if (!outimg->add_plane(heif_channel_R, width, height, 8) ||
-      !outimg->add_plane(heif_channel_G, width, height, 8) ||
-      !outimg->add_plane(heif_channel_B, width, height, 8)) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_R, width, height, 8, limits) ||
+                 outimg->add_plane(heif_channel_G, width, height, 8, limits) ||
+                 outimg->add_plane(heif_channel_B, width, height, 8, limits)) {
+    return err;
   }
 
   if (want_alpha) {
-    if (!outimg->add_plane(heif_channel_Alpha, width, height, 8)) {
-      return nullptr;
+    if (auto err = outimg->add_plane(heif_channel_Alpha, width, height, 8, limits)) {
+      return err;
     }
   }
 
   const uint8_t* in_p;
-  int in_p_stride = 0;
+  size_t in_p_stride = 0;
   int in_pix_size = has_alpha ? 4 : 3;
 
   uint8_t* out_r, * out_g, * out_b, * out_a = nullptr;
-  int out_r_stride = 0, out_g_stride = 0, out_b_stride = 0, out_a_stride = 0;
+  size_t out_r_stride = 0, out_g_stride = 0, out_b_stride = 0, out_a_stride = 0;
 
   in_p = input->get_plane(heif_channel_interleaved, &in_p_stride);
 
@@ -591,7 +610,7 @@ Op_RGB24_32_to_RGB::convert_colorspace(const std::shared_ptr<const HeifPixelImag
     out_a = outimg->get_plane(heif_channel_Alpha, &out_a_stride);
   }
 
-  int x, y;
+  uint32_t x, y;
   for (y = 0; y < height; y++) {
 
     for (x = 0; x < width; x++) {
@@ -613,7 +632,8 @@ Op_RGB24_32_to_RGB::convert_colorspace(const std::shared_ptr<const HeifPixelImag
 std::vector<ColorStateWithCost>
 Op_RRGGBBaa_swap_endianness::state_after_conversion(const ColorState& input_state,
                                                     const ColorState& target_state,
-                                                    const heif_color_conversion_options& options) const
+                                                    const heif_color_conversion_options& options,
+                                                    const heif_color_conversion_options_ext& options_ext) const
 {
   // Note: no input alpha channel required. It will be filled up with 0xFF.
 
@@ -645,7 +665,7 @@ Op_RRGGBBaa_swap_endianness::state_after_conversion(const ColorState& input_stat
     output_state.has_alpha = false;
     output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-    states.push_back({output_state, SpeedCosts_Unoptimized});
+    states.emplace_back(output_state, SpeedCosts_Unoptimized);
   }
 
 
@@ -665,24 +685,25 @@ Op_RRGGBBaa_swap_endianness::state_after_conversion(const ColorState& input_stat
     output_state.has_alpha = true;
     output_state.bits_per_pixel = input_state.bits_per_pixel;
 
-    states.push_back({output_state, SpeedCosts_Unoptimized});
+    states.emplace_back(output_state, SpeedCosts_Unoptimized);
   }
 
-
   return states;
 }
 
 
-std::shared_ptr<HeifPixelImage>
+Result<std::shared_ptr<HeifPixelImage>>
 Op_RRGGBBaa_swap_endianness::convert_colorspace(const std::shared_ptr<const HeifPixelImage>& input,
                                                 const ColorState& input_state,
                                                 const ColorState& target_state,
-                                                const heif_color_conversion_options& options) const
+                                                const heif_color_conversion_options& options,
+                                                const heif_color_conversion_options_ext& options_ext,
+                                                const heif_security_limits* limits) const
 {
   auto outimg = std::make_shared<HeifPixelImage>();
 
-  int width = input->get_width();
-  int height = input->get_height();
+  uint32_t width = input->get_width();
+  uint32_t height = input->get_height();
 
   switch (input->get_chroma_format()) {
     case heif_chroma_interleaved_RRGGBB_LE:
@@ -698,26 +719,26 @@ Op_RRGGBBaa_swap_endianness::convert_colorspace(const std::shared_ptr<const Heif
       outimg->create(width, height, heif_colorspace_RGB, heif_chroma_interleaved_RRGGBBAA_LE);
       break;
     default:
-      return nullptr;
+      return Error::InternalError;
   }
 
-  if (!outimg->add_plane(heif_channel_interleaved, width, height,
-                         input->get_bits_per_pixel(heif_channel_interleaved))) {
-    return nullptr;
+  if (auto err = outimg->add_plane(heif_channel_interleaved, width, height,
+                                   input->get_bits_per_pixel(heif_channel_interleaved), limits)) {
+    return err;
   }
 
   const uint8_t* in_p = nullptr;
-  int in_p_stride = 0;
+  size_t in_p_stride = 0;
 
   uint8_t* out_p;
-  int out_p_stride = 0;
+  size_t out_p_stride = 0;
 
   in_p = input->get_plane(heif_channel_interleaved, &in_p_stride);
   out_p = outimg->get_plane(heif_channel_interleaved, &out_p_stride);
 
-  int n_bytes = std::min(in_p_stride, out_p_stride);
+  size_t n_bytes = std::min(in_p_stride, out_p_stride);
 
-  int x, y;
+  uint32_t x, y;
   for (y = 0; y < height; y++) {
     for (x = 0; x < n_bytes; x += 2) {
       out_p[y * out_p_stride + x + 0] = in_p[y * in_p_stride + x + 1];
@@ -727,5 +748,3 @@ Op_RRGGBBaa_swap_endianness::convert_colorspace(const std::shared_ptr<const Heif
 
   return outimg;
 }
-
-
