@@ -20,9 +20,9 @@
 
 
 #include "nclx.h"
+#include "libheif/heif_experimental.h"
 
 #include <cassert>
-#include <limits>
 #include <memory>
 #include <string>
 #include <vector>
@@ -220,7 +220,7 @@ Error color_profile_nclx::parse(BitstreamRange& range)
 {
   StreamReader::grow_status status;
   status = range.wait_for_available_bytes(7);
-  if (status != StreamReader::size_reached) {
+  if (status != StreamReader::grow_status::size_reached) {
     // TODO: return recoverable error at timeout
     return Error(heif_error_Invalid_input,
                  heif_suberror_End_of_data);
@@ -294,9 +294,11 @@ struct heif_color_profile_nclx* color_profile_nclx::alloc_nclx_color_profile()
 
   if (profile) {
     profile->version = 1;
-    profile->color_primaries = heif_color_primaries_unspecified;
-    profile->transfer_characteristics = heif_transfer_characteristic_unspecified;
-    profile->matrix_coefficients = heif_matrix_coefficients_ITU_R_BT_601_6;
+
+    // sRGB defaults
+    profile->color_primaries = heif_color_primaries_ITU_R_BT_709_5; // 1
+    profile->transfer_characteristics = heif_transfer_characteristic_IEC_61966_2_1; // 13
+    profile->matrix_coefficients = heif_matrix_coefficients_ITU_R_BT_601_6; // 6
     profile->full_range_flag = true;
   }
 
@@ -310,10 +312,11 @@ void color_profile_nclx::free_nclx_color_profile(struct heif_color_profile_nclx*
 }
 
 
-void color_profile_nclx::set_default()
+void color_profile_nclx::set_sRGB_defaults()
 {
-  m_colour_primaries = 2;
-  m_transfer_characteristics = 2;
+  // sRGB defaults
+  m_colour_primaries = 1;
+  m_transfer_characteristics = 13;
   m_matrix_coefficients = 6;
   m_full_range_flag = true;
 }
@@ -339,7 +342,23 @@ void color_profile_nclx::set_from_heif_color_profile_nclx(const struct heif_colo
 }
 
 
-Error Box_colr::parse(BitstreamRange& range)
+void color_profile_nclx::replace_undefined_values_with_sRGB_defaults()
+{
+  if (m_matrix_coefficients == heif_matrix_coefficients_unspecified) {
+    m_matrix_coefficients = heif_matrix_coefficients_ITU_R_BT_601_6;
+  }
+
+  if (m_colour_primaries == heif_color_primaries_unspecified) {
+    m_colour_primaries = heif_color_primaries_ITU_R_BT_709_5;
+  }
+
+  if (m_transfer_characteristics == heif_transfer_characteristic_unspecified) {
+    m_transfer_characteristics = heif_transfer_characteristic_IEC_61966_2_1;
+  }
+}
+
+
+Error Box_colr::parse(BitstreamRange& range, const heif_security_limits* limits)
 {
   StreamReader::grow_status status;
   uint32_t colour_type = range.read32();
@@ -354,15 +373,19 @@ Error Box_colr::parse(BitstreamRange& range)
   }
   else if (colour_type == fourcc("prof") ||
            colour_type == fourcc("rICC")) {
+    if (!has_fixed_box_size()) {
+      return Error(heif_error_Unsupported_feature, heif_suberror_Unspecified, "colr boxes with undefined box size are not supported");
+    }
+
     uint64_t profile_size_64 = get_box_size() - get_header_size() - 4;
-    if (profile_size_64 > std::numeric_limits<size_t>::max()) {
+    if (limits->max_color_profile_size && profile_size_64 > limits->max_color_profile_size) {
       return Error(heif_error_Invalid_input, heif_suberror_Security_limit_exceeded, "Color profile exceeds maximum supported size");
     }
 
     size_t profile_size = static_cast<size_t>(profile_size_64);
 
     status = range.wait_for_available_bytes(profile_size);
-    if (status != StreamReader::size_reached) {
+    if (status != StreamReader::grow_status::size_reached) {
       // TODO: return recoverable error at timeout
       return Error(heif_error_Invalid_input,
                    heif_suberror_End_of_data);
@@ -390,7 +413,7 @@ std::string Box_colr::dump(Indent& indent) const
   sstr << Box::dump(indent);
 
   if (m_color_profile) {
-    sstr << indent << "colour_type: " << to_fourcc(get_color_profile_type()) << "\n";
+    sstr << indent << "colour_type: " << fourcc_to_string(get_color_profile_type()) << "\n";
     sstr << m_color_profile->dump(indent);
   }
   else {
