@@ -22,7 +22,7 @@
 #ifndef LIBHEIF_ENCODER_OPENJPEG_H
 #define LIBHEIF_ENCODER_OPENJPEG_H
 
-#include "libheif/common_utils.h"
+#include "common_utils.h"
 
 
 const struct heif_encoder_plugin* get_encoder_plugin_openjpeg();
