@@ -22,7 +22,7 @@ set -e
 
 ROOT="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"
 
-INSTALL_PACKAGES=
+INSTALL_PACKAGES="gdb "
 REMOVE_PACKAGES=
 BUILD_ROOT=$ROOT/..
 UPDATE_APT=
@@ -120,16 +120,14 @@ fi
 
 if [ "$MINGW" == "32" ]; then
     sudo dpkg --add-architecture i386
-    # https://github.com/actions/runner-images/issues/4589
-    sudo rm -f /etc/apt/sources.list.d/microsoft-prod.list
     sudo apt-get update
-    sudo apt-get install -y --allow-downgrades libgd3:i386/jammy libpcre2-8-0:i386/jammy libpcre2-16-0:i386/jammy libpcre2-32-0:i386/jammy libpcre2-posix3/jammy
-    sudo apt-get purge -y libmono* moby* mono* php* libgdiplus libpcre2-posix3 libzip4
     INSTALL_PACKAGES="$INSTALL_PACKAGES \
         binutils-mingw-w64-i686 \
         g++-mingw-w64-i686 \
         gcc-mingw-w64-i686 \
         mingw-w64-i686-dev \
+        libz-mingw-w64-dev \
+        libz-mingw-w64 \
         wine-stable \
         wine32 \
         "
@@ -139,6 +137,8 @@ elif [ "$MINGW" == "64" ]; then
         g++-mingw-w64-x86-64 \
         gcc-mingw-w64-x86-64 \
         mingw-w64-x86-64-dev \
+        libz-mingw-w64-dev \
+        libz-mingw-w64 \
         wine-stable \
         "
 fi
