@@ -271,6 +271,7 @@ static bool NclxMatches(heif_colorspace colorspace,
 void TestConversion(const std::string& test_name, const ColorState& input_state,
                     const ColorState& target_state,
                     const heif_color_conversion_options& options,
+                    const heif_color_conversion_options_ext& options_ext,
                     bool require_supported = true) {
   INFO(test_name);
   INFO("downsampling=" << options.preferred_chroma_downsampling_algorithm
@@ -280,7 +281,7 @@ void TestConversion(const std::string& test_name, const ColorState& input_state,
                        << (bool)options.only_use_preferred_chroma_algorithm);
 
   ColorConversionPipeline pipeline;
-  bool supported = pipeline.construct_pipeline(input_state, target_state, options);
+  bool supported = pipeline.construct_pipeline(input_state, target_state, options, options_ext);
   if (require_supported) REQUIRE(supported);
   if (!supported) return;
   INFO("conversion pipeline: " << pipeline.debug_dump_pipeline());
@@ -317,7 +318,7 @@ void TestConversion(const std::string& test_name, const ColorState& input_state,
 
   // Convert back in the other direction (if supported).
   ColorConversionPipeline reverse_pipeline;
-  if (reverse_pipeline.construct_pipeline(target_state, input_state, options)) {
+  if (reverse_pipeline.construct_pipeline(target_state, input_state, options, options_ext)) {
     INFO("reverse pipeline: " << reverse_pipeline.debug_dump_pipeline());
     auto recovered_image_result =reverse_pipeline.convert_image(out_image, heif_get_disabled_security_limits());
     REQUIRE(recovered_image_result);
@@ -362,7 +363,8 @@ void TestConversion(const std::string& test_name, const ColorState& input_state,
 void TestFailingConversion(const std::string& test_name,
                            const ColorState& input_state,
                            const ColorState& target_state,
-                           const heif_color_conversion_options& options = {}) {
+                           const heif_color_conversion_options& options,
+                           const heif_color_conversion_options_ext& options_ext) {
   INFO(test_name);
   INFO("downsampling=" << options.preferred_chroma_downsampling_algorithm
                        << " upsampling="
@@ -371,7 +373,7 @@ void TestFailingConversion(const std::string& test_name,
                        << (bool)options.only_use_preferred_chroma_algorithm);
   ColorConversionPipeline pipeline;
   bool construct_pipeline_res =
-      pipeline.construct_pipeline(input_state, target_state, options);
+      pipeline.construct_pipeline(input_state, target_state, options, options_ext);
   INFO("conversion pipeline: " << pipeline.debug_dump_pipeline());
   REQUIRE_FALSE(construct_pipeline_res);
 }
@@ -485,6 +487,10 @@ TEST_CASE("All conversions", "[heif_image]") {
       .preferred_chroma_upsampling_algorithm = upsampling,
       .only_use_preferred_chroma_algorithm = only_use_preferred_chroma_algorithm};
 
+  heif_color_conversion_options_ext options_ext = {
+      .alpha_composition_mode = heif_alpha_composition_mode_none
+  };
+
   // Test all source and destination state combinations.
   ColorState src_state = GENERATE(
       from_range(GetAllColorStates(GetSupportedMatrices())));
@@ -510,7 +516,7 @@ TEST_CASE("All conversions", "[heif_image]") {
 
   std::ostringstream os;
   os << "from: " << src_state << "\nto:   " << dst_state;
-  TestConversion(os.str(), src_state, dst_state, options, require_supported);
+  TestConversion(os.str(), src_state, dst_state, options, options_ext, require_supported);
 }
 
 TEST_CASE("Unsupported matrices", "[heif_image]") {
@@ -531,6 +537,10 @@ TEST_CASE("Unsupported matrices", "[heif_image]") {
       .preferred_chroma_upsampling_algorithm = upsampling,
       .only_use_preferred_chroma_algorithm = only_use_preferred_chroma_algorithm};
 
+  heif_color_conversion_options_ext options_ext = {
+      .alpha_composition_mode = heif_alpha_composition_mode_none
+  };
+
   ColorState src_state =
       GENERATE(from_range(GetAllColorStates(GetUnsupportedMatrices())));
   ColorState dst_state =
@@ -547,7 +557,7 @@ TEST_CASE("Unsupported matrices", "[heif_image]") {
 
   std::ostringstream os;
   os << "from: " << src_state << "\nto:   " << dst_state;
-  TestFailingConversion(os.str(), src_state, dst_state, options);
+  TestFailingConversion(os.str(), src_state, dst_state, options, options_ext);
 }
 
 TEST_CASE("Sharp yuv conversion", "[heif_image]") {
@@ -557,53 +567,57 @@ TEST_CASE("Sharp yuv conversion", "[heif_image]") {
       .preferred_chroma_upsampling_algorithm = heif_chroma_upsampling_bilinear,
       .only_use_preferred_chroma_algorithm = true};
 
+  heif_color_conversion_options_ext options_ext = {
+      .alpha_composition_mode = heif_alpha_composition_mode_none
+  };
+
 #ifdef HAVE_LIBSHARPYUV
   TestConversion("### interleaved RGBA -> YCbCr 420 with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_interleaved_RGBA, true, 8},
                  {heif_colorspace_YCbCr, heif_chroma_420, true, 8},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### interleaved RGB 10bit -> YCbCr 420 10bit with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_interleaved_RGB, false, 8},
                  {heif_colorspace_YCbCr, heif_chroma_420, false, 8},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
 
   TestConversion("### interleaved RGBA 12bit big endian -> YCbCr 420 12bit with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_interleaved_RRGGBBAA_BE, true, 12},
                  {heif_colorspace_YCbCr, heif_chroma_420, true, 12},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### interleaved RGBA 12bit little endian -> YCbCr 420 12bit with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_interleaved_RRGGBBAA_LE, true, 12},
                  {heif_colorspace_YCbCr, heif_chroma_420, true, 12},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### planar RGB -> YCbCr 420 with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_444, false, 8},
                  {heif_colorspace_YCbCr, heif_chroma_420, false, 8},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### planar RGBA -> YCbCr 420 with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_444, true, 8},
                  {heif_colorspace_YCbCr, heif_chroma_420, true, 8},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### planar RGB 10bit -> YCbCr 420 10bit with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_444, false, 10},
                  {heif_colorspace_YCbCr, heif_chroma_420, false, 10},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
   TestConversion("### planar RGBA 10bit -> YCbCr 420 10bit with sharp yuv",
                  {heif_colorspace_RGB, heif_chroma_444, true, 10},
                  {heif_colorspace_YCbCr, heif_chroma_420, true, 10},
-                 sharp_yuv_options);
+                 sharp_yuv_options, options_ext);
 #else
   // Should fail if libsharpyuv is not compiled in.
   TestFailingConversion(
       "### interleaved RGBA -> YCbCr 420 with sharp yuv NOT COMPILED IN",
       {heif_colorspace_RGB, heif_chroma_interleaved_RGBA, true, 8},
-      {heif_colorspace_YCbCr, heif_chroma_420, false, 8}, sharp_yuv_options);
+      {heif_colorspace_YCbCr, heif_chroma_420, false, 8}, sharp_yuv_options, options_ext);
   WARN("Tests built without sharp yuv");
 #endif
 
   TestFailingConversion(
       "### interleaved RGBA -> YCbCr 422 with sharp yuv (not supported!)",
       {heif_colorspace_RGB, heif_chroma_interleaved_RGBA, true, 8},
-      {heif_colorspace_YCbCr, heif_chroma_422, false, 8}, sharp_yuv_options);
+      {heif_colorspace_YCbCr, heif_chroma_422, false, 8}, sharp_yuv_options, options_ext);
 }
 
 
@@ -661,7 +675,7 @@ TEST_CASE("Bilinear upsampling", "[heif_image]")
              {255, 200,
               50, 0});
 
-  auto conversionResult = convert_colorspace(img, heif_colorspace_YCbCr, heif_chroma_444, nullptr, 8, options, heif_get_disabled_security_limits());
+  auto conversionResult = convert_colorspace(img, heif_colorspace_YCbCr, heif_chroma_444, nullptr, 8, options, nullptr, heif_get_disabled_security_limits());
   REQUIRE(conversionResult);
   std::shared_ptr<HeifPixelImage> out = *conversionResult;
 
@@ -714,7 +728,7 @@ TEST_CASE("RGB 5-6-5 to RGB")
     }
   }
 
-  auto conversionResult = convert_colorspace(img, heif_colorspace_RGB, heif_chroma_444, nullptr, 8, options, heif_get_disabled_security_limits());
+  auto conversionResult = convert_colorspace(img, heif_colorspace_RGB, heif_chroma_444, nullptr, 8, options, nullptr, heif_get_disabled_security_limits());
   REQUIRE(conversionResult);
   std::shared_ptr<HeifPixelImage> out = *conversionResult;
 
