@@ -24,9 +24,9 @@
   SOFTWARE.
 */
 
-#include "catch.hpp"
+#include "catch_amalgamated.hpp"
 #include "libheif/heif.h"
-#include "codecs/jpeg2000.h"
+#include "codecs/jpeg2000_boxes.h"
 #include <cstdint>
 #include <iostream>
 
