@@ -25,7 +25,7 @@
 */
 #include "catch_amalgamated.hpp"
 #include "libheif/heif.h"
-#include "libheif/api_structs.h"
+#include "api_structs.h"
 #include <cstdint>
 #include <stdio.h>
 #include "test_utils.h"
