@@ -25,7 +25,7 @@
 */
 
 #include "catch_amalgamated.hpp"
-#include "libheif/api_structs.h"
+#include "api_structs.h"
 #include "libheif/heif.h"
 #include <cstdint>
 #include <string.h>
